/*
 * This file is part of the GROMACS molecular simulation package.
 *
 * Copyright (c) 2012,2013,2014,2015, by the GROMACS development team, led by
 * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
 * and including many others, as listed in the AUTHORS file in the
 * top-level source directory and at http://www.gromacs.org.
 *
 * GROMACS is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * GROMACS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GROMACS; if not, see
 * http://www.gnu.org/licenses, or write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * If you want to redistribute modifications to GROMACS, please
 * consider that scientific software is very special. Version
 * control is crucial - bugs must be traceable. We will be happy to
 * consider code for inclusion in the official distribution, but
 * derived work must not be called official GROMACS. Details are found
 * in the README & COPYING files - if they are missing, get the
 * official version at http://www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the research papers on the package. Check out http://www.gromacs.org.
 */
/*
 * Note: this file was generated by the GROMACS avx_256_double kernel generator.
 */
#include "gmxpre.h"

#include "config.h"

#include <math.h>

#include "../nb_kernel.h"
#include "gromacs/gmxlib/nrnb.h"

#include "kernelutil_x86_avx_256_double.h"

/*
 * Gromacs nonbonded kernel:   nb_kernel_ElecCoul_VdwCSTab_GeomW3W3_VF_avx_256_double
 * Electrostatics interaction: Coulomb
 * VdW interaction:            CubicSplineTable
 * Geometry:                   Water3-Water3
 * Calculate force/pot:        PotentialAndForce
 */
void
nb_kernel_ElecCoul_VdwCSTab_GeomW3W3_VF_avx_256_double
                    (t_nblist                    * gmx_restrict       nlist,
                     rvec                        * gmx_restrict          xx,
                     rvec                        * gmx_restrict          ff,
                     t_forcerec                  * gmx_restrict          fr,
                     t_mdatoms                   * gmx_restrict     mdatoms,
                     nb_kernel_data_t gmx_unused * gmx_restrict kernel_data,
                     t_nrnb                      * gmx_restrict        nrnb)
{
    /* Suffixes 0,1,2,3 refer to particle indices for waters in the inner or outer loop, or 
     * just 0 for non-waters.
     * Suffixes A,B,C,D refer to j loop unrolling done with AVX, e.g. for the four different
     * jnr indices corresponding to data put in the four positions in the SIMD register.
     */
    int              i_shift_offset,i_coord_offset,outeriter,inneriter;
    int              j_index_start,j_index_end,jidx,nri,inr,ggid,iidx;
    int              jnrA,jnrB,jnrC,jnrD;
    int              jnrlistA,jnrlistB,jnrlistC,jnrlistD;
    int              jnrlistE,jnrlistF,jnrlistG,jnrlistH;
    int              j_coord_offsetA,j_coord_offsetB,j_coord_offsetC,j_coord_offsetD;
    int              *iinr,*jindex,*jjnr,*shiftidx,*gid;
    real             rcutoff_scalar;
    real             *shiftvec,*fshift,*x,*f;
    real             *fjptrA,*fjptrB,*fjptrC,*fjptrD;
    real             scratch[4*DIM];
    __m256d          tx,ty,tz,fscal,rcutoff,rcutoff2,jidxall;
    real *           vdwioffsetptr0;
    __m256d          ix0,iy0,iz0,fix0,fiy0,fiz0,iq0,isai0;
    real *           vdwioffsetptr1;
    __m256d          ix1,iy1,iz1,fix1,fiy1,fiz1,iq1,isai1;
    real *           vdwioffsetptr2;
    __m256d          ix2,iy2,iz2,fix2,fiy2,fiz2,iq2,isai2;
    int              vdwjidx0A,vdwjidx0B,vdwjidx0C,vdwjidx0D;
    __m256d          jx0,jy0,jz0,fjx0,fjy0,fjz0,jq0,isaj0;
    int              vdwjidx1A,vdwjidx1B,vdwjidx1C,vdwjidx1D;
    __m256d          jx1,jy1,jz1,fjx1,fjy1,fjz1,jq1,isaj1;
    int              vdwjidx2A,vdwjidx2B,vdwjidx2C,vdwjidx2D;
    __m256d          jx2,jy2,jz2,fjx2,fjy2,fjz2,jq2,isaj2;
    __m256d          dx00,dy00,dz00,rsq00,rinv00,rinvsq00,r00,qq00,c6_00,c12_00;
    __m256d          dx01,dy01,dz01,rsq01,rinv01,rinvsq01,r01,qq01,c6_01,c12_01;
    __m256d          dx02,dy02,dz02,rsq02,rinv02,rinvsq02,r02,qq02,c6_02,c12_02;
    __m256d          dx10,dy10,dz10,rsq10,rinv10,rinvsq10,r10,qq10,c6_10,c12_10;
    __m256d          dx11,dy11,dz11,rsq11,rinv11,rinvsq11,r11,qq11,c6_11,c12_11;
    __m256d          dx12,dy12,dz12,rsq12,rinv12,rinvsq12,r12,qq12,c6_12,c12_12;
    __m256d          dx20,dy20,dz20,rsq20,rinv20,rinvsq20,r20,qq20,c6_20,c12_20;
    __m256d          dx21,dy21,dz21,rsq21,rinv21,rinvsq21,r21,qq21,c6_21,c12_21;
    __m256d          dx22,dy22,dz22,rsq22,rinv22,rinvsq22,r22,qq22,c6_22,c12_22;
    __m256d          velec,felec,velecsum,facel,crf,krf,krf2;
    real             *charge;
    int              nvdwtype;
    __m256d          rinvsix,rvdw,vvdw,vvdw6,vvdw12,fvdw,fvdw6,fvdw12,vvdwsum,sh_vdw_invrcut6;
    int              *vdwtype;
    real             *vdwparam;
    __m256d          one_sixth   = _mm256_set1_pd(1.0/6.0);
    __m256d          one_twelfth = _mm256_set1_pd(1.0/12.0);
    __m128i          vfitab;
    __m128i          ifour       = _mm_set1_epi32(4);
    __m256d          rt,vfeps,vftabscale,Y,F,G,H,Heps,Fp,VV,FF;
    real             *vftab;
    __m256d          dummy_mask,cutoff_mask;
    __m128           tmpmask0,tmpmask1;
    __m256d          signbit = _mm256_castsi256_pd( _mm256_set1_epi32(0x80000000) );
    __m256d          one     = _mm256_set1_pd(1.0);
    __m256d          two     = _mm256_set1_pd(2.0);
    x                = xx[0];
    f                = ff[0];

    nri              = nlist->nri;
    iinr             = nlist->iinr;
    jindex           = nlist->jindex;
    jjnr             = nlist->jjnr;
    shiftidx         = nlist->shift;
    gid              = nlist->gid;
    shiftvec         = fr->shift_vec[0];
    fshift           = fr->fshift[0];
    facel            = _mm256_set1_pd(fr->epsfac);
    charge           = mdatoms->chargeA;
    nvdwtype         = fr->ntype;
    vdwparam         = fr->nbfp;
    vdwtype          = mdatoms->typeA;

    vftab            = kernel_data->table_vdw->data;
    vftabscale       = _mm256_set1_pd(kernel_data->table_vdw->scale);

    /* Setup water-specific parameters */
    inr              = nlist->iinr[0];
    iq0              = _mm256_mul_pd(facel,_mm256_set1_pd(charge[inr+0]));
    iq1              = _mm256_mul_pd(facel,_mm256_set1_pd(charge[inr+1]));
    iq2              = _mm256_mul_pd(facel,_mm256_set1_pd(charge[inr+2]));
    vdwioffsetptr0   = vdwparam+2*nvdwtype*vdwtype[inr+0];

    jq0              = _mm256_set1_pd(charge[inr+0]);
    jq1              = _mm256_set1_pd(charge[inr+1]);
    jq2              = _mm256_set1_pd(charge[inr+2]);
    vdwjidx0A        = 2*vdwtype[inr+0];
    qq00             = _mm256_mul_pd(iq0,jq0);
    c6_00            = _mm256_set1_pd(vdwioffsetptr0[vdwjidx0A]);
    c12_00           = _mm256_set1_pd(vdwioffsetptr0[vdwjidx0A+1]);
    qq01             = _mm256_mul_pd(iq0,jq1);
    qq02             = _mm256_mul_pd(iq0,jq2);
    qq10             = _mm256_mul_pd(iq1,jq0);
    qq11             = _mm256_mul_pd(iq1,jq1);
    qq12             = _mm256_mul_pd(iq1,jq2);
    qq20             = _mm256_mul_pd(iq2,jq0);
    qq21             = _mm256_mul_pd(iq2,jq1);
    qq22             = _mm256_mul_pd(iq2,jq2);

    /* Avoid stupid compiler warnings */
    jnrA = jnrB = jnrC = jnrD = 0;
    j_coord_offsetA = 0;
    j_coord_offsetB = 0;
    j_coord_offsetC = 0;
    j_coord_offsetD = 0;

    outeriter        = 0;
    inneriter        = 0;

    for(iidx=0;iidx<4*DIM;iidx++)
    {
        scratch[iidx] = 0.0;
    }

    /* Start outer loop over neighborlists */
    for(iidx=0; iidx<nri; iidx++)
    {
        /* Load shift vector for this list */
        i_shift_offset   = DIM*shiftidx[iidx];

        /* Load limits for loop over neighbors */
        j_index_start    = jindex[iidx];
        j_index_end      = jindex[iidx+1];

        /* Get outer coordinate index */
        inr              = iinr[iidx];
        i_coord_offset   = DIM*inr;

        /* Load i particle coords and add shift vector */
        gmx_mm256_load_shift_and_3rvec_broadcast_pd(shiftvec+i_shift_offset,x+i_coord_offset,
                                                    &ix0,&iy0,&iz0,&ix1,&iy1,&iz1,&ix2,&iy2,&iz2);

        fix0             = _mm256_setzero_pd();
        fiy0             = _mm256_setzero_pd();
        fiz0             = _mm256_setzero_pd();
        fix1             = _mm256_setzero_pd();
        fiy1             = _mm256_setzero_pd();
        fiz1             = _mm256_setzero_pd();
        fix2             = _mm256_setzero_pd();
        fiy2             = _mm256_setzero_pd();
        fiz2             = _mm256_setzero_pd();

        /* Reset potential sums */
        velecsum         = _mm256_setzero_pd();
        vvdwsum          = _mm256_setzero_pd();

        /* Start inner kernel loop */
        for(jidx=j_index_start; jidx<j_index_end && jjnr[jidx+3]>=0; jidx+=4)
        {

            /* Get j neighbor index, and coordinate index */
            jnrA             = jjnr[jidx];
            jnrB             = jjnr[jidx+1];
            jnrC             = jjnr[jidx+2];
            jnrD             = jjnr[jidx+3];
            j_coord_offsetA  = DIM*jnrA;
            j_coord_offsetB  = DIM*jnrB;
            j_coord_offsetC  = DIM*jnrC;
            j_coord_offsetD  = DIM*jnrD;

            /* load j atom coordinates */
            gmx_mm256_load_3rvec_4ptr_swizzle_pd(x+j_coord_offsetA,x+j_coord_offsetB,
                                                 x+j_coord_offsetC,x+j_coord_offsetD,
                                              &jx0,&jy0,&jz0,&jx1,&jy1,&jz1,&jx2,&jy2,&jz2);

            /* Calculate displacement vector */
            dx00             = _mm256_sub_pd(ix0,jx0);
            dy00             = _mm256_sub_pd(iy0,jy0);
            dz00             = _mm256_sub_pd(iz0,jz0);
            dx01             = _mm256_sub_pd(ix0,jx1);
            dy01             = _mm256_sub_pd(iy0,jy1);
            dz01             = _mm256_sub_pd(iz0,jz1);
            dx02             = _mm256_sub_pd(ix0,jx2);
            dy02             = _mm256_sub_pd(iy0,jy2);
            dz02             = _mm256_sub_pd(iz0,jz2);
            dx10             = _mm256_sub_pd(ix1,jx0);
            dy10             = _mm256_sub_pd(iy1,jy0);
            dz10             = _mm256_sub_pd(iz1,jz0);
            dx11             = _mm256_sub_pd(ix1,jx1);
            dy11             = _mm256_sub_pd(iy1,jy1);
            dz11             = _mm256_sub_pd(iz1,jz1);
            dx12             = _mm256_sub_pd(ix1,jx2);
            dy12             = _mm256_sub_pd(iy1,jy2);
            dz12             = _mm256_sub_pd(iz1,jz2);
            dx20             = _mm256_sub_pd(ix2,jx0);
            dy20             = _mm256_sub_pd(iy2,jy0);
            dz20             = _mm256_sub_pd(iz2,jz0);
            dx21             = _mm256_sub_pd(ix2,jx1);
            dy21             = _mm256_sub_pd(iy2,jy1);
            dz21             = _mm256_sub_pd(iz2,jz1);
            dx22             = _mm256_sub_pd(ix2,jx2);
            dy22             = _mm256_sub_pd(iy2,jy2);
            dz22             = _mm256_sub_pd(iz2,jz2);

            /* Calculate squared distance and things based on it */
            rsq00            = gmx_mm256_calc_rsq_pd(dx00,dy00,dz00);
            rsq01            = gmx_mm256_calc_rsq_pd(dx01,dy01,dz01);
            rsq02            = gmx_mm256_calc_rsq_pd(dx02,dy02,dz02);
            rsq10            = gmx_mm256_calc_rsq_pd(dx10,dy10,dz10);
            rsq11            = gmx_mm256_calc_rsq_pd(dx11,dy11,dz11);
            rsq12            = gmx_mm256_calc_rsq_pd(dx12,dy12,dz12);
            rsq20            = gmx_mm256_calc_rsq_pd(dx20,dy20,dz20);
            rsq21            = gmx_mm256_calc_rsq_pd(dx21,dy21,dz21);
            rsq22            = gmx_mm256_calc_rsq_pd(dx22,dy22,dz22);

            rinv00           = avx256_invsqrt_d(rsq00);
            rinv01           = avx256_invsqrt_d(rsq01);
            rinv02           = avx256_invsqrt_d(rsq02);
            rinv10           = avx256_invsqrt_d(rsq10);
            rinv11           = avx256_invsqrt_d(rsq11);
            rinv12           = avx256_invsqrt_d(rsq12);
            rinv20           = avx256_invsqrt_d(rsq20);
            rinv21           = avx256_invsqrt_d(rsq21);
            rinv22           = avx256_invsqrt_d(rsq22);

            rinvsq00         = _mm256_mul_pd(rinv00,rinv00);
            rinvsq01         = _mm256_mul_pd(rinv01,rinv01);
            rinvsq02         = _mm256_mul_pd(rinv02,rinv02);
            rinvsq10         = _mm256_mul_pd(rinv10,rinv10);
            rinvsq11         = _mm256_mul_pd(rinv11,rinv11);
            rinvsq12         = _mm256_mul_pd(rinv12,rinv12);
            rinvsq20         = _mm256_mul_pd(rinv20,rinv20);
            rinvsq21         = _mm256_mul_pd(rinv21,rinv21);
            rinvsq22         = _mm256_mul_pd(rinv22,rinv22);

            fjx0             = _mm256_setzero_pd();
            fjy0             = _mm256_setzero_pd();
            fjz0             = _mm256_setzero_pd();
            fjx1             = _mm256_setzero_pd();
            fjy1             = _mm256_setzero_pd();
            fjz1             = _mm256_setzero_pd();
            fjx2             = _mm256_setzero_pd();
            fjy2             = _mm256_setzero_pd();
            fjz2             = _mm256_setzero_pd();

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            r00              = _mm256_mul_pd(rsq00,rinv00);

            /* Calculate table index by multiplying r with table scale and truncate to integer */
            rt               = _mm256_mul_pd(r00,vftabscale);
            vfitab           = _mm256_cvttpd_epi32(rt);
            vfeps            = _mm256_sub_pd(rt,_mm256_round_pd(rt, _MM_FROUND_FLOOR));
            vfitab           = _mm_slli_epi32(vfitab,3);

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq00,rinv00);
            felec            = _mm256_mul_pd(velec,rinvsq00);

            /* CUBIC SPLINE TABLE DISPERSION */
            Y                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,0) );
            F                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,1) );
            G                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,2) );
            H                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,3) );
            GMX_MM256_FULLTRANSPOSE4_PD(Y,F,G,H);
            Heps             = _mm256_mul_pd(vfeps,H);
            Fp               = _mm256_add_pd(F,_mm256_mul_pd(vfeps,_mm256_add_pd(G,Heps)));
            VV               = _mm256_add_pd(Y,_mm256_mul_pd(vfeps,Fp));
            vvdw6            = _mm256_mul_pd(c6_00,VV);
            FF               = _mm256_add_pd(Fp,_mm256_mul_pd(vfeps,_mm256_add_pd(G,_mm256_add_pd(Heps,Heps))));
            fvdw6            = _mm256_mul_pd(c6_00,FF);

            /* CUBIC SPLINE TABLE REPULSION */
            vfitab           = _mm_add_epi32(vfitab,ifour);
            Y                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,0) );
            F                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,1) );
            G                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,2) );
            H                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,3) );
            GMX_MM256_FULLTRANSPOSE4_PD(Y,F,G,H);
            Heps             = _mm256_mul_pd(vfeps,H);
            Fp               = _mm256_add_pd(F,_mm256_mul_pd(vfeps,_mm256_add_pd(G,Heps)));
            VV               = _mm256_add_pd(Y,_mm256_mul_pd(vfeps,Fp));
            vvdw12           = _mm256_mul_pd(c12_00,VV);
            FF               = _mm256_add_pd(Fp,_mm256_mul_pd(vfeps,_mm256_add_pd(G,_mm256_add_pd(Heps,Heps))));
            fvdw12           = _mm256_mul_pd(c12_00,FF);
            vvdw             = _mm256_add_pd(vvdw12,vvdw6);
            fvdw             = _mm256_xor_pd(signbit,_mm256_mul_pd(_mm256_add_pd(fvdw6,fvdw12),_mm256_mul_pd(vftabscale,rinv00)));

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);
            vvdwsum          = _mm256_add_pd(vvdwsum,vvdw);

            fscal            = _mm256_add_pd(felec,fvdw);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx00);
            ty               = _mm256_mul_pd(fscal,dy00);
            tz               = _mm256_mul_pd(fscal,dz00);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq01,rinv01);
            felec            = _mm256_mul_pd(velec,rinvsq01);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx01);
            ty               = _mm256_mul_pd(fscal,dy01);
            tz               = _mm256_mul_pd(fscal,dz01);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq02,rinv02);
            felec            = _mm256_mul_pd(velec,rinvsq02);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx02);
            ty               = _mm256_mul_pd(fscal,dy02);
            tz               = _mm256_mul_pd(fscal,dz02);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq10,rinv10);
            felec            = _mm256_mul_pd(velec,rinvsq10);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx10);
            ty               = _mm256_mul_pd(fscal,dy10);
            tz               = _mm256_mul_pd(fscal,dz10);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq11,rinv11);
            felec            = _mm256_mul_pd(velec,rinvsq11);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx11);
            ty               = _mm256_mul_pd(fscal,dy11);
            tz               = _mm256_mul_pd(fscal,dz11);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq12,rinv12);
            felec            = _mm256_mul_pd(velec,rinvsq12);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx12);
            ty               = _mm256_mul_pd(fscal,dy12);
            tz               = _mm256_mul_pd(fscal,dz12);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq20,rinv20);
            felec            = _mm256_mul_pd(velec,rinvsq20);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx20);
            ty               = _mm256_mul_pd(fscal,dy20);
            tz               = _mm256_mul_pd(fscal,dz20);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq21,rinv21);
            felec            = _mm256_mul_pd(velec,rinvsq21);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx21);
            ty               = _mm256_mul_pd(fscal,dy21);
            tz               = _mm256_mul_pd(fscal,dz21);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq22,rinv22);
            felec            = _mm256_mul_pd(velec,rinvsq22);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx22);
            ty               = _mm256_mul_pd(fscal,dy22);
            tz               = _mm256_mul_pd(fscal,dz22);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            fjptrA             = f+j_coord_offsetA;
            fjptrB             = f+j_coord_offsetB;
            fjptrC             = f+j_coord_offsetC;
            fjptrD             = f+j_coord_offsetD;

            gmx_mm256_decrement_3rvec_4ptr_swizzle_pd(fjptrA,fjptrB,fjptrC,fjptrD,
                                                      fjx0,fjy0,fjz0,fjx1,fjy1,fjz1,fjx2,fjy2,fjz2);

            /* Inner loop uses 278 flops */
        }

        if(jidx<j_index_end)
        {

            /* Get j neighbor index, and coordinate index */
            jnrlistA         = jjnr[jidx];
            jnrlistB         = jjnr[jidx+1];
            jnrlistC         = jjnr[jidx+2];
            jnrlistD         = jjnr[jidx+3];
            /* Sign of each element will be negative for non-real atoms.
             * This mask will be 0xFFFFFFFF for dummy entries and 0x0 for real ones,
             * so use it as val = _mm_andnot_pd(mask,val) to clear dummy entries.
             */
            tmpmask0 = gmx_mm_castsi128_ps(_mm_cmplt_epi32(_mm_loadu_si128((const __m128i *)(jjnr+jidx)),_mm_setzero_si128()));

            tmpmask1 = _mm_permute_ps(tmpmask0,_GMX_MM_PERMUTE(3,3,2,2));
            tmpmask0 = _mm_permute_ps(tmpmask0,_GMX_MM_PERMUTE(1,1,0,0));
            dummy_mask = _mm256_castps_pd(gmx_mm256_set_m128(tmpmask1,tmpmask0));

            jnrA       = (jnrlistA>=0) ? jnrlistA : 0;
            jnrB       = (jnrlistB>=0) ? jnrlistB : 0;
            jnrC       = (jnrlistC>=0) ? jnrlistC : 0;
            jnrD       = (jnrlistD>=0) ? jnrlistD : 0;
            j_coord_offsetA  = DIM*jnrA;
            j_coord_offsetB  = DIM*jnrB;
            j_coord_offsetC  = DIM*jnrC;
            j_coord_offsetD  = DIM*jnrD;

            /* load j atom coordinates */
            gmx_mm256_load_3rvec_4ptr_swizzle_pd(x+j_coord_offsetA,x+j_coord_offsetB,
                                                 x+j_coord_offsetC,x+j_coord_offsetD,
                                              &jx0,&jy0,&jz0,&jx1,&jy1,&jz1,&jx2,&jy2,&jz2);

            /* Calculate displacement vector */
            dx00             = _mm256_sub_pd(ix0,jx0);
            dy00             = _mm256_sub_pd(iy0,jy0);
            dz00             = _mm256_sub_pd(iz0,jz0);
            dx01             = _mm256_sub_pd(ix0,jx1);
            dy01             = _mm256_sub_pd(iy0,jy1);
            dz01             = _mm256_sub_pd(iz0,jz1);
            dx02             = _mm256_sub_pd(ix0,jx2);
            dy02             = _mm256_sub_pd(iy0,jy2);
            dz02             = _mm256_sub_pd(iz0,jz2);
            dx10             = _mm256_sub_pd(ix1,jx0);
            dy10             = _mm256_sub_pd(iy1,jy0);
            dz10             = _mm256_sub_pd(iz1,jz0);
            dx11             = _mm256_sub_pd(ix1,jx1);
            dy11             = _mm256_sub_pd(iy1,jy1);
            dz11             = _mm256_sub_pd(iz1,jz1);
            dx12             = _mm256_sub_pd(ix1,jx2);
            dy12             = _mm256_sub_pd(iy1,jy2);
            dz12             = _mm256_sub_pd(iz1,jz2);
            dx20             = _mm256_sub_pd(ix2,jx0);
            dy20             = _mm256_sub_pd(iy2,jy0);
            dz20             = _mm256_sub_pd(iz2,jz0);
            dx21             = _mm256_sub_pd(ix2,jx1);
            dy21             = _mm256_sub_pd(iy2,jy1);
            dz21             = _mm256_sub_pd(iz2,jz1);
            dx22             = _mm256_sub_pd(ix2,jx2);
            dy22             = _mm256_sub_pd(iy2,jy2);
            dz22             = _mm256_sub_pd(iz2,jz2);

            /* Calculate squared distance and things based on it */
            rsq00            = gmx_mm256_calc_rsq_pd(dx00,dy00,dz00);
            rsq01            = gmx_mm256_calc_rsq_pd(dx01,dy01,dz01);
            rsq02            = gmx_mm256_calc_rsq_pd(dx02,dy02,dz02);
            rsq10            = gmx_mm256_calc_rsq_pd(dx10,dy10,dz10);
            rsq11            = gmx_mm256_calc_rsq_pd(dx11,dy11,dz11);
            rsq12            = gmx_mm256_calc_rsq_pd(dx12,dy12,dz12);
            rsq20            = gmx_mm256_calc_rsq_pd(dx20,dy20,dz20);
            rsq21            = gmx_mm256_calc_rsq_pd(dx21,dy21,dz21);
            rsq22            = gmx_mm256_calc_rsq_pd(dx22,dy22,dz22);

            rinv00           = avx256_invsqrt_d(rsq00);
            rinv01           = avx256_invsqrt_d(rsq01);
            rinv02           = avx256_invsqrt_d(rsq02);
            rinv10           = avx256_invsqrt_d(rsq10);
            rinv11           = avx256_invsqrt_d(rsq11);
            rinv12           = avx256_invsqrt_d(rsq12);
            rinv20           = avx256_invsqrt_d(rsq20);
            rinv21           = avx256_invsqrt_d(rsq21);
            rinv22           = avx256_invsqrt_d(rsq22);

            rinvsq00         = _mm256_mul_pd(rinv00,rinv00);
            rinvsq01         = _mm256_mul_pd(rinv01,rinv01);
            rinvsq02         = _mm256_mul_pd(rinv02,rinv02);
            rinvsq10         = _mm256_mul_pd(rinv10,rinv10);
            rinvsq11         = _mm256_mul_pd(rinv11,rinv11);
            rinvsq12         = _mm256_mul_pd(rinv12,rinv12);
            rinvsq20         = _mm256_mul_pd(rinv20,rinv20);
            rinvsq21         = _mm256_mul_pd(rinv21,rinv21);
            rinvsq22         = _mm256_mul_pd(rinv22,rinv22);

            fjx0             = _mm256_setzero_pd();
            fjy0             = _mm256_setzero_pd();
            fjz0             = _mm256_setzero_pd();
            fjx1             = _mm256_setzero_pd();
            fjy1             = _mm256_setzero_pd();
            fjz1             = _mm256_setzero_pd();
            fjx2             = _mm256_setzero_pd();
            fjy2             = _mm256_setzero_pd();
            fjz2             = _mm256_setzero_pd();

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            r00              = _mm256_mul_pd(rsq00,rinv00);
            r00              = _mm256_andnot_pd(dummy_mask,r00);

            /* Calculate table index by multiplying r with table scale and truncate to integer */
            rt               = _mm256_mul_pd(r00,vftabscale);
            vfitab           = _mm256_cvttpd_epi32(rt);
            vfeps            = _mm256_sub_pd(rt,_mm256_round_pd(rt, _MM_FROUND_FLOOR));
            vfitab           = _mm_slli_epi32(vfitab,3);

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq00,rinv00);
            felec            = _mm256_mul_pd(velec,rinvsq00);

            /* CUBIC SPLINE TABLE DISPERSION */
            Y                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,0) );
            F                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,1) );
            G                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,2) );
            H                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,3) );
            GMX_MM256_FULLTRANSPOSE4_PD(Y,F,G,H);
            Heps             = _mm256_mul_pd(vfeps,H);
            Fp               = _mm256_add_pd(F,_mm256_mul_pd(vfeps,_mm256_add_pd(G,Heps)));
            VV               = _mm256_add_pd(Y,_mm256_mul_pd(vfeps,Fp));
            vvdw6            = _mm256_mul_pd(c6_00,VV);
            FF               = _mm256_add_pd(Fp,_mm256_mul_pd(vfeps,_mm256_add_pd(G,_mm256_add_pd(Heps,Heps))));
            fvdw6            = _mm256_mul_pd(c6_00,FF);

            /* CUBIC SPLINE TABLE REPULSION */
            vfitab           = _mm_add_epi32(vfitab,ifour);
            Y                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,0) );
            F                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,1) );
            G                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,2) );
            H                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,3) );
            GMX_MM256_FULLTRANSPOSE4_PD(Y,F,G,H);
            Heps             = _mm256_mul_pd(vfeps,H);
            Fp               = _mm256_add_pd(F,_mm256_mul_pd(vfeps,_mm256_add_pd(G,Heps)));
            VV               = _mm256_add_pd(Y,_mm256_mul_pd(vfeps,Fp));
            vvdw12           = _mm256_mul_pd(c12_00,VV);
            FF               = _mm256_add_pd(Fp,_mm256_mul_pd(vfeps,_mm256_add_pd(G,_mm256_add_pd(Heps,Heps))));
            fvdw12           = _mm256_mul_pd(c12_00,FF);
            vvdw             = _mm256_add_pd(vvdw12,vvdw6);
            fvdw             = _mm256_xor_pd(signbit,_mm256_mul_pd(_mm256_add_pd(fvdw6,fvdw12),_mm256_mul_pd(vftabscale,rinv00)));

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);
            vvdw             = _mm256_andnot_pd(dummy_mask,vvdw);
            vvdwsum          = _mm256_add_pd(vvdwsum,vvdw);

            fscal            = _mm256_add_pd(felec,fvdw);

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx00);
            ty               = _mm256_mul_pd(fscal,dy00);
            tz               = _mm256_mul_pd(fscal,dz00);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq01,rinv01);
            felec            = _mm256_mul_pd(velec,rinvsq01);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx01);
            ty               = _mm256_mul_pd(fscal,dy01);
            tz               = _mm256_mul_pd(fscal,dz01);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq02,rinv02);
            felec            = _mm256_mul_pd(velec,rinvsq02);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx02);
            ty               = _mm256_mul_pd(fscal,dy02);
            tz               = _mm256_mul_pd(fscal,dz02);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq10,rinv10);
            felec            = _mm256_mul_pd(velec,rinvsq10);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx10);
            ty               = _mm256_mul_pd(fscal,dy10);
            tz               = _mm256_mul_pd(fscal,dz10);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq11,rinv11);
            felec            = _mm256_mul_pd(velec,rinvsq11);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx11);
            ty               = _mm256_mul_pd(fscal,dy11);
            tz               = _mm256_mul_pd(fscal,dz11);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq12,rinv12);
            felec            = _mm256_mul_pd(velec,rinvsq12);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx12);
            ty               = _mm256_mul_pd(fscal,dy12);
            tz               = _mm256_mul_pd(fscal,dz12);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq20,rinv20);
            felec            = _mm256_mul_pd(velec,rinvsq20);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx20);
            ty               = _mm256_mul_pd(fscal,dy20);
            tz               = _mm256_mul_pd(fscal,dz20);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq21,rinv21);
            felec            = _mm256_mul_pd(velec,rinvsq21);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx21);
            ty               = _mm256_mul_pd(fscal,dy21);
            tz               = _mm256_mul_pd(fscal,dz21);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq22,rinv22);
            felec            = _mm256_mul_pd(velec,rinvsq22);

            /* Update potential sum for this i atom from the interaction with this j atom. */
            velec            = _mm256_andnot_pd(dummy_mask,velec);
            velecsum         = _mm256_add_pd(velecsum,velec);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx22);
            ty               = _mm256_mul_pd(fscal,dy22);
            tz               = _mm256_mul_pd(fscal,dz22);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            fjptrA             = (jnrlistA>=0) ? f+j_coord_offsetA : scratch;
            fjptrB             = (jnrlistB>=0) ? f+j_coord_offsetB : scratch;
            fjptrC             = (jnrlistC>=0) ? f+j_coord_offsetC : scratch;
            fjptrD             = (jnrlistD>=0) ? f+j_coord_offsetD : scratch;

            gmx_mm256_decrement_3rvec_4ptr_swizzle_pd(fjptrA,fjptrB,fjptrC,fjptrD,
                                                      fjx0,fjy0,fjz0,fjx1,fjy1,fjz1,fjx2,fjy2,fjz2);

            /* Inner loop uses 279 flops */
        }

        /* End of innermost loop */

        gmx_mm256_update_iforce_3atom_swizzle_pd(fix0,fiy0,fiz0,fix1,fiy1,fiz1,fix2,fiy2,fiz2,
                                                 f+i_coord_offset,fshift+i_shift_offset);

        ggid                        = gid[iidx];
        /* Update potential energies */
        gmx_mm256_update_1pot_pd(velecsum,kernel_data->energygrp_elec+ggid);
        gmx_mm256_update_1pot_pd(vvdwsum,kernel_data->energygrp_vdw+ggid);

        /* Increment number of inner iterations */
        inneriter                  += j_index_end - j_index_start;

        /* Outer loop uses 20 flops */
    }

    /* Increment number of outer iterations */
    outeriter        += nri;

    /* Update outer/inner flops */

    inc_nrnb(nrnb,eNR_NBKERNEL_ELEC_VDW_W3W3_VF,outeriter*20 + inneriter*279);
}
/*
 * Gromacs nonbonded kernel:   nb_kernel_ElecCoul_VdwCSTab_GeomW3W3_F_avx_256_double
 * Electrostatics interaction: Coulomb
 * VdW interaction:            CubicSplineTable
 * Geometry:                   Water3-Water3
 * Calculate force/pot:        Force
 */
void
nb_kernel_ElecCoul_VdwCSTab_GeomW3W3_F_avx_256_double
                    (t_nblist                    * gmx_restrict       nlist,
                     rvec                        * gmx_restrict          xx,
                     rvec                        * gmx_restrict          ff,
                     t_forcerec                  * gmx_restrict          fr,
                     t_mdatoms                   * gmx_restrict     mdatoms,
                     nb_kernel_data_t gmx_unused * gmx_restrict kernel_data,
                     t_nrnb                      * gmx_restrict        nrnb)
{
    /* Suffixes 0,1,2,3 refer to particle indices for waters in the inner or outer loop, or 
     * just 0 for non-waters.
     * Suffixes A,B,C,D refer to j loop unrolling done with AVX, e.g. for the four different
     * jnr indices corresponding to data put in the four positions in the SIMD register.
     */
    int              i_shift_offset,i_coord_offset,outeriter,inneriter;
    int              j_index_start,j_index_end,jidx,nri,inr,ggid,iidx;
    int              jnrA,jnrB,jnrC,jnrD;
    int              jnrlistA,jnrlistB,jnrlistC,jnrlistD;
    int              jnrlistE,jnrlistF,jnrlistG,jnrlistH;
    int              j_coord_offsetA,j_coord_offsetB,j_coord_offsetC,j_coord_offsetD;
    int              *iinr,*jindex,*jjnr,*shiftidx,*gid;
    real             rcutoff_scalar;
    real             *shiftvec,*fshift,*x,*f;
    real             *fjptrA,*fjptrB,*fjptrC,*fjptrD;
    real             scratch[4*DIM];
    __m256d          tx,ty,tz,fscal,rcutoff,rcutoff2,jidxall;
    real *           vdwioffsetptr0;
    __m256d          ix0,iy0,iz0,fix0,fiy0,fiz0,iq0,isai0;
    real *           vdwioffsetptr1;
    __m256d          ix1,iy1,iz1,fix1,fiy1,fiz1,iq1,isai1;
    real *           vdwioffsetptr2;
    __m256d          ix2,iy2,iz2,fix2,fiy2,fiz2,iq2,isai2;
    int              vdwjidx0A,vdwjidx0B,vdwjidx0C,vdwjidx0D;
    __m256d          jx0,jy0,jz0,fjx0,fjy0,fjz0,jq0,isaj0;
    int              vdwjidx1A,vdwjidx1B,vdwjidx1C,vdwjidx1D;
    __m256d          jx1,jy1,jz1,fjx1,fjy1,fjz1,jq1,isaj1;
    int              vdwjidx2A,vdwjidx2B,vdwjidx2C,vdwjidx2D;
    __m256d          jx2,jy2,jz2,fjx2,fjy2,fjz2,jq2,isaj2;
    __m256d          dx00,dy00,dz00,rsq00,rinv00,rinvsq00,r00,qq00,c6_00,c12_00;
    __m256d          dx01,dy01,dz01,rsq01,rinv01,rinvsq01,r01,qq01,c6_01,c12_01;
    __m256d          dx02,dy02,dz02,rsq02,rinv02,rinvsq02,r02,qq02,c6_02,c12_02;
    __m256d          dx10,dy10,dz10,rsq10,rinv10,rinvsq10,r10,qq10,c6_10,c12_10;
    __m256d          dx11,dy11,dz11,rsq11,rinv11,rinvsq11,r11,qq11,c6_11,c12_11;
    __m256d          dx12,dy12,dz12,rsq12,rinv12,rinvsq12,r12,qq12,c6_12,c12_12;
    __m256d          dx20,dy20,dz20,rsq20,rinv20,rinvsq20,r20,qq20,c6_20,c12_20;
    __m256d          dx21,dy21,dz21,rsq21,rinv21,rinvsq21,r21,qq21,c6_21,c12_21;
    __m256d          dx22,dy22,dz22,rsq22,rinv22,rinvsq22,r22,qq22,c6_22,c12_22;
    __m256d          velec,felec,velecsum,facel,crf,krf,krf2;
    real             *charge;
    int              nvdwtype;
    __m256d          rinvsix,rvdw,vvdw,vvdw6,vvdw12,fvdw,fvdw6,fvdw12,vvdwsum,sh_vdw_invrcut6;
    int              *vdwtype;
    real             *vdwparam;
    __m256d          one_sixth   = _mm256_set1_pd(1.0/6.0);
    __m256d          one_twelfth = _mm256_set1_pd(1.0/12.0);
    __m128i          vfitab;
    __m128i          ifour       = _mm_set1_epi32(4);
    __m256d          rt,vfeps,vftabscale,Y,F,G,H,Heps,Fp,VV,FF;
    real             *vftab;
    __m256d          dummy_mask,cutoff_mask;
    __m128           tmpmask0,tmpmask1;
    __m256d          signbit = _mm256_castsi256_pd( _mm256_set1_epi32(0x80000000) );
    __m256d          one     = _mm256_set1_pd(1.0);
    __m256d          two     = _mm256_set1_pd(2.0);
    x                = xx[0];
    f                = ff[0];

    nri              = nlist->nri;
    iinr             = nlist->iinr;
    jindex           = nlist->jindex;
    jjnr             = nlist->jjnr;
    shiftidx         = nlist->shift;
    gid              = nlist->gid;
    shiftvec         = fr->shift_vec[0];
    fshift           = fr->fshift[0];
    facel            = _mm256_set1_pd(fr->epsfac);
    charge           = mdatoms->chargeA;
    nvdwtype         = fr->ntype;
    vdwparam         = fr->nbfp;
    vdwtype          = mdatoms->typeA;

    vftab            = kernel_data->table_vdw->data;
    vftabscale       = _mm256_set1_pd(kernel_data->table_vdw->scale);

    /* Setup water-specific parameters */
    inr              = nlist->iinr[0];
    iq0              = _mm256_mul_pd(facel,_mm256_set1_pd(charge[inr+0]));
    iq1              = _mm256_mul_pd(facel,_mm256_set1_pd(charge[inr+1]));
    iq2              = _mm256_mul_pd(facel,_mm256_set1_pd(charge[inr+2]));
    vdwioffsetptr0   = vdwparam+2*nvdwtype*vdwtype[inr+0];

    jq0              = _mm256_set1_pd(charge[inr+0]);
    jq1              = _mm256_set1_pd(charge[inr+1]);
    jq2              = _mm256_set1_pd(charge[inr+2]);
    vdwjidx0A        = 2*vdwtype[inr+0];
    qq00             = _mm256_mul_pd(iq0,jq0);
    c6_00            = _mm256_set1_pd(vdwioffsetptr0[vdwjidx0A]);
    c12_00           = _mm256_set1_pd(vdwioffsetptr0[vdwjidx0A+1]);
    qq01             = _mm256_mul_pd(iq0,jq1);
    qq02             = _mm256_mul_pd(iq0,jq2);
    qq10             = _mm256_mul_pd(iq1,jq0);
    qq11             = _mm256_mul_pd(iq1,jq1);
    qq12             = _mm256_mul_pd(iq1,jq2);
    qq20             = _mm256_mul_pd(iq2,jq0);
    qq21             = _mm256_mul_pd(iq2,jq1);
    qq22             = _mm256_mul_pd(iq2,jq2);

    /* Avoid stupid compiler warnings */
    jnrA = jnrB = jnrC = jnrD = 0;
    j_coord_offsetA = 0;
    j_coord_offsetB = 0;
    j_coord_offsetC = 0;
    j_coord_offsetD = 0;

    outeriter        = 0;
    inneriter        = 0;

    for(iidx=0;iidx<4*DIM;iidx++)
    {
        scratch[iidx] = 0.0;
    }

    /* Start outer loop over neighborlists */
    for(iidx=0; iidx<nri; iidx++)
    {
        /* Load shift vector for this list */
        i_shift_offset   = DIM*shiftidx[iidx];

        /* Load limits for loop over neighbors */
        j_index_start    = jindex[iidx];
        j_index_end      = jindex[iidx+1];

        /* Get outer coordinate index */
        inr              = iinr[iidx];
        i_coord_offset   = DIM*inr;

        /* Load i particle coords and add shift vector */
        gmx_mm256_load_shift_and_3rvec_broadcast_pd(shiftvec+i_shift_offset,x+i_coord_offset,
                                                    &ix0,&iy0,&iz0,&ix1,&iy1,&iz1,&ix2,&iy2,&iz2);

        fix0             = _mm256_setzero_pd();
        fiy0             = _mm256_setzero_pd();
        fiz0             = _mm256_setzero_pd();
        fix1             = _mm256_setzero_pd();
        fiy1             = _mm256_setzero_pd();
        fiz1             = _mm256_setzero_pd();
        fix2             = _mm256_setzero_pd();
        fiy2             = _mm256_setzero_pd();
        fiz2             = _mm256_setzero_pd();

        /* Start inner kernel loop */
        for(jidx=j_index_start; jidx<j_index_end && jjnr[jidx+3]>=0; jidx+=4)
        {

            /* Get j neighbor index, and coordinate index */
            jnrA             = jjnr[jidx];
            jnrB             = jjnr[jidx+1];
            jnrC             = jjnr[jidx+2];
            jnrD             = jjnr[jidx+3];
            j_coord_offsetA  = DIM*jnrA;
            j_coord_offsetB  = DIM*jnrB;
            j_coord_offsetC  = DIM*jnrC;
            j_coord_offsetD  = DIM*jnrD;

            /* load j atom coordinates */
            gmx_mm256_load_3rvec_4ptr_swizzle_pd(x+j_coord_offsetA,x+j_coord_offsetB,
                                                 x+j_coord_offsetC,x+j_coord_offsetD,
                                              &jx0,&jy0,&jz0,&jx1,&jy1,&jz1,&jx2,&jy2,&jz2);

            /* Calculate displacement vector */
            dx00             = _mm256_sub_pd(ix0,jx0);
            dy00             = _mm256_sub_pd(iy0,jy0);
            dz00             = _mm256_sub_pd(iz0,jz0);
            dx01             = _mm256_sub_pd(ix0,jx1);
            dy01             = _mm256_sub_pd(iy0,jy1);
            dz01             = _mm256_sub_pd(iz0,jz1);
            dx02             = _mm256_sub_pd(ix0,jx2);
            dy02             = _mm256_sub_pd(iy0,jy2);
            dz02             = _mm256_sub_pd(iz0,jz2);
            dx10             = _mm256_sub_pd(ix1,jx0);
            dy10             = _mm256_sub_pd(iy1,jy0);
            dz10             = _mm256_sub_pd(iz1,jz0);
            dx11             = _mm256_sub_pd(ix1,jx1);
            dy11             = _mm256_sub_pd(iy1,jy1);
            dz11             = _mm256_sub_pd(iz1,jz1);
            dx12             = _mm256_sub_pd(ix1,jx2);
            dy12             = _mm256_sub_pd(iy1,jy2);
            dz12             = _mm256_sub_pd(iz1,jz2);
            dx20             = _mm256_sub_pd(ix2,jx0);
            dy20             = _mm256_sub_pd(iy2,jy0);
            dz20             = _mm256_sub_pd(iz2,jz0);
            dx21             = _mm256_sub_pd(ix2,jx1);
            dy21             = _mm256_sub_pd(iy2,jy1);
            dz21             = _mm256_sub_pd(iz2,jz1);
            dx22             = _mm256_sub_pd(ix2,jx2);
            dy22             = _mm256_sub_pd(iy2,jy2);
            dz22             = _mm256_sub_pd(iz2,jz2);

            /* Calculate squared distance and things based on it */
            rsq00            = gmx_mm256_calc_rsq_pd(dx00,dy00,dz00);
            rsq01            = gmx_mm256_calc_rsq_pd(dx01,dy01,dz01);
            rsq02            = gmx_mm256_calc_rsq_pd(dx02,dy02,dz02);
            rsq10            = gmx_mm256_calc_rsq_pd(dx10,dy10,dz10);
            rsq11            = gmx_mm256_calc_rsq_pd(dx11,dy11,dz11);
            rsq12            = gmx_mm256_calc_rsq_pd(dx12,dy12,dz12);
            rsq20            = gmx_mm256_calc_rsq_pd(dx20,dy20,dz20);
            rsq21            = gmx_mm256_calc_rsq_pd(dx21,dy21,dz21);
            rsq22            = gmx_mm256_calc_rsq_pd(dx22,dy22,dz22);

            rinv00           = avx256_invsqrt_d(rsq00);
            rinv01           = avx256_invsqrt_d(rsq01);
            rinv02           = avx256_invsqrt_d(rsq02);
            rinv10           = avx256_invsqrt_d(rsq10);
            rinv11           = avx256_invsqrt_d(rsq11);
            rinv12           = avx256_invsqrt_d(rsq12);
            rinv20           = avx256_invsqrt_d(rsq20);
            rinv21           = avx256_invsqrt_d(rsq21);
            rinv22           = avx256_invsqrt_d(rsq22);

            rinvsq00         = _mm256_mul_pd(rinv00,rinv00);
            rinvsq01         = _mm256_mul_pd(rinv01,rinv01);
            rinvsq02         = _mm256_mul_pd(rinv02,rinv02);
            rinvsq10         = _mm256_mul_pd(rinv10,rinv10);
            rinvsq11         = _mm256_mul_pd(rinv11,rinv11);
            rinvsq12         = _mm256_mul_pd(rinv12,rinv12);
            rinvsq20         = _mm256_mul_pd(rinv20,rinv20);
            rinvsq21         = _mm256_mul_pd(rinv21,rinv21);
            rinvsq22         = _mm256_mul_pd(rinv22,rinv22);

            fjx0             = _mm256_setzero_pd();
            fjy0             = _mm256_setzero_pd();
            fjz0             = _mm256_setzero_pd();
            fjx1             = _mm256_setzero_pd();
            fjy1             = _mm256_setzero_pd();
            fjz1             = _mm256_setzero_pd();
            fjx2             = _mm256_setzero_pd();
            fjy2             = _mm256_setzero_pd();
            fjz2             = _mm256_setzero_pd();

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            r00              = _mm256_mul_pd(rsq00,rinv00);

            /* Calculate table index by multiplying r with table scale and truncate to integer */
            rt               = _mm256_mul_pd(r00,vftabscale);
            vfitab           = _mm256_cvttpd_epi32(rt);
            vfeps            = _mm256_sub_pd(rt,_mm256_round_pd(rt, _MM_FROUND_FLOOR));
            vfitab           = _mm_slli_epi32(vfitab,3);

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq00,rinv00);
            felec            = _mm256_mul_pd(velec,rinvsq00);

            /* CUBIC SPLINE TABLE DISPERSION */
            Y                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,0) );
            F                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,1) );
            G                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,2) );
            H                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,3) );
            GMX_MM256_FULLTRANSPOSE4_PD(Y,F,G,H);
            Heps             = _mm256_mul_pd(vfeps,H);
            Fp               = _mm256_add_pd(F,_mm256_mul_pd(vfeps,_mm256_add_pd(G,Heps)));
            FF               = _mm256_add_pd(Fp,_mm256_mul_pd(vfeps,_mm256_add_pd(G,_mm256_add_pd(Heps,Heps))));
            fvdw6            = _mm256_mul_pd(c6_00,FF);

            /* CUBIC SPLINE TABLE REPULSION */
            vfitab           = _mm_add_epi32(vfitab,ifour);
            Y                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,0) );
            F                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,1) );
            G                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,2) );
            H                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,3) );
            GMX_MM256_FULLTRANSPOSE4_PD(Y,F,G,H);
            Heps             = _mm256_mul_pd(vfeps,H);
            Fp               = _mm256_add_pd(F,_mm256_mul_pd(vfeps,_mm256_add_pd(G,Heps)));
            FF               = _mm256_add_pd(Fp,_mm256_mul_pd(vfeps,_mm256_add_pd(G,_mm256_add_pd(Heps,Heps))));
            fvdw12           = _mm256_mul_pd(c12_00,FF);
            fvdw             = _mm256_xor_pd(signbit,_mm256_mul_pd(_mm256_add_pd(fvdw6,fvdw12),_mm256_mul_pd(vftabscale,rinv00)));

            fscal            = _mm256_add_pd(felec,fvdw);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx00);
            ty               = _mm256_mul_pd(fscal,dy00);
            tz               = _mm256_mul_pd(fscal,dz00);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq01,rinv01);
            felec            = _mm256_mul_pd(velec,rinvsq01);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx01);
            ty               = _mm256_mul_pd(fscal,dy01);
            tz               = _mm256_mul_pd(fscal,dz01);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq02,rinv02);
            felec            = _mm256_mul_pd(velec,rinvsq02);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx02);
            ty               = _mm256_mul_pd(fscal,dy02);
            tz               = _mm256_mul_pd(fscal,dz02);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq10,rinv10);
            felec            = _mm256_mul_pd(velec,rinvsq10);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx10);
            ty               = _mm256_mul_pd(fscal,dy10);
            tz               = _mm256_mul_pd(fscal,dz10);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq11,rinv11);
            felec            = _mm256_mul_pd(velec,rinvsq11);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx11);
            ty               = _mm256_mul_pd(fscal,dy11);
            tz               = _mm256_mul_pd(fscal,dz11);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq12,rinv12);
            felec            = _mm256_mul_pd(velec,rinvsq12);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx12);
            ty               = _mm256_mul_pd(fscal,dy12);
            tz               = _mm256_mul_pd(fscal,dz12);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq20,rinv20);
            felec            = _mm256_mul_pd(velec,rinvsq20);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx20);
            ty               = _mm256_mul_pd(fscal,dy20);
            tz               = _mm256_mul_pd(fscal,dz20);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq21,rinv21);
            felec            = _mm256_mul_pd(velec,rinvsq21);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx21);
            ty               = _mm256_mul_pd(fscal,dy21);
            tz               = _mm256_mul_pd(fscal,dz21);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq22,rinv22);
            felec            = _mm256_mul_pd(velec,rinvsq22);

            fscal            = felec;

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx22);
            ty               = _mm256_mul_pd(fscal,dy22);
            tz               = _mm256_mul_pd(fscal,dz22);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            fjptrA             = f+j_coord_offsetA;
            fjptrB             = f+j_coord_offsetB;
            fjptrC             = f+j_coord_offsetC;
            fjptrD             = f+j_coord_offsetD;

            gmx_mm256_decrement_3rvec_4ptr_swizzle_pd(fjptrA,fjptrB,fjptrC,fjptrD,
                                                      fjx0,fjy0,fjz0,fjx1,fjy1,fjz1,fjx2,fjy2,fjz2);

            /* Inner loop uses 261 flops */
        }

        if(jidx<j_index_end)
        {

            /* Get j neighbor index, and coordinate index */
            jnrlistA         = jjnr[jidx];
            jnrlistB         = jjnr[jidx+1];
            jnrlistC         = jjnr[jidx+2];
            jnrlistD         = jjnr[jidx+3];
            /* Sign of each element will be negative for non-real atoms.
             * This mask will be 0xFFFFFFFF for dummy entries and 0x0 for real ones,
             * so use it as val = _mm_andnot_pd(mask,val) to clear dummy entries.
             */
            tmpmask0 = gmx_mm_castsi128_ps(_mm_cmplt_epi32(_mm_loadu_si128((const __m128i *)(jjnr+jidx)),_mm_setzero_si128()));

            tmpmask1 = _mm_permute_ps(tmpmask0,_GMX_MM_PERMUTE(3,3,2,2));
            tmpmask0 = _mm_permute_ps(tmpmask0,_GMX_MM_PERMUTE(1,1,0,0));
            dummy_mask = _mm256_castps_pd(gmx_mm256_set_m128(tmpmask1,tmpmask0));

            jnrA       = (jnrlistA>=0) ? jnrlistA : 0;
            jnrB       = (jnrlistB>=0) ? jnrlistB : 0;
            jnrC       = (jnrlistC>=0) ? jnrlistC : 0;
            jnrD       = (jnrlistD>=0) ? jnrlistD : 0;
            j_coord_offsetA  = DIM*jnrA;
            j_coord_offsetB  = DIM*jnrB;
            j_coord_offsetC  = DIM*jnrC;
            j_coord_offsetD  = DIM*jnrD;

            /* load j atom coordinates */
            gmx_mm256_load_3rvec_4ptr_swizzle_pd(x+j_coord_offsetA,x+j_coord_offsetB,
                                                 x+j_coord_offsetC,x+j_coord_offsetD,
                                              &jx0,&jy0,&jz0,&jx1,&jy1,&jz1,&jx2,&jy2,&jz2);

            /* Calculate displacement vector */
            dx00             = _mm256_sub_pd(ix0,jx0);
            dy00             = _mm256_sub_pd(iy0,jy0);
            dz00             = _mm256_sub_pd(iz0,jz0);
            dx01             = _mm256_sub_pd(ix0,jx1);
            dy01             = _mm256_sub_pd(iy0,jy1);
            dz01             = _mm256_sub_pd(iz0,jz1);
            dx02             = _mm256_sub_pd(ix0,jx2);
            dy02             = _mm256_sub_pd(iy0,jy2);
            dz02             = _mm256_sub_pd(iz0,jz2);
            dx10             = _mm256_sub_pd(ix1,jx0);
            dy10             = _mm256_sub_pd(iy1,jy0);
            dz10             = _mm256_sub_pd(iz1,jz0);
            dx11             = _mm256_sub_pd(ix1,jx1);
            dy11             = _mm256_sub_pd(iy1,jy1);
            dz11             = _mm256_sub_pd(iz1,jz1);
            dx12             = _mm256_sub_pd(ix1,jx2);
            dy12             = _mm256_sub_pd(iy1,jy2);
            dz12             = _mm256_sub_pd(iz1,jz2);
            dx20             = _mm256_sub_pd(ix2,jx0);
            dy20             = _mm256_sub_pd(iy2,jy0);
            dz20             = _mm256_sub_pd(iz2,jz0);
            dx21             = _mm256_sub_pd(ix2,jx1);
            dy21             = _mm256_sub_pd(iy2,jy1);
            dz21             = _mm256_sub_pd(iz2,jz1);
            dx22             = _mm256_sub_pd(ix2,jx2);
            dy22             = _mm256_sub_pd(iy2,jy2);
            dz22             = _mm256_sub_pd(iz2,jz2);

            /* Calculate squared distance and things based on it */
            rsq00            = gmx_mm256_calc_rsq_pd(dx00,dy00,dz00);
            rsq01            = gmx_mm256_calc_rsq_pd(dx01,dy01,dz01);
            rsq02            = gmx_mm256_calc_rsq_pd(dx02,dy02,dz02);
            rsq10            = gmx_mm256_calc_rsq_pd(dx10,dy10,dz10);
            rsq11            = gmx_mm256_calc_rsq_pd(dx11,dy11,dz11);
            rsq12            = gmx_mm256_calc_rsq_pd(dx12,dy12,dz12);
            rsq20            = gmx_mm256_calc_rsq_pd(dx20,dy20,dz20);
            rsq21            = gmx_mm256_calc_rsq_pd(dx21,dy21,dz21);
            rsq22            = gmx_mm256_calc_rsq_pd(dx22,dy22,dz22);

            rinv00           = avx256_invsqrt_d(rsq00);
            rinv01           = avx256_invsqrt_d(rsq01);
            rinv02           = avx256_invsqrt_d(rsq02);
            rinv10           = avx256_invsqrt_d(rsq10);
            rinv11           = avx256_invsqrt_d(rsq11);
            rinv12           = avx256_invsqrt_d(rsq12);
            rinv20           = avx256_invsqrt_d(rsq20);
            rinv21           = avx256_invsqrt_d(rsq21);
            rinv22           = avx256_invsqrt_d(rsq22);

            rinvsq00         = _mm256_mul_pd(rinv00,rinv00);
            rinvsq01         = _mm256_mul_pd(rinv01,rinv01);
            rinvsq02         = _mm256_mul_pd(rinv02,rinv02);
            rinvsq10         = _mm256_mul_pd(rinv10,rinv10);
            rinvsq11         = _mm256_mul_pd(rinv11,rinv11);
            rinvsq12         = _mm256_mul_pd(rinv12,rinv12);
            rinvsq20         = _mm256_mul_pd(rinv20,rinv20);
            rinvsq21         = _mm256_mul_pd(rinv21,rinv21);
            rinvsq22         = _mm256_mul_pd(rinv22,rinv22);

            fjx0             = _mm256_setzero_pd();
            fjy0             = _mm256_setzero_pd();
            fjz0             = _mm256_setzero_pd();
            fjx1             = _mm256_setzero_pd();
            fjy1             = _mm256_setzero_pd();
            fjz1             = _mm256_setzero_pd();
            fjx2             = _mm256_setzero_pd();
            fjy2             = _mm256_setzero_pd();
            fjz2             = _mm256_setzero_pd();

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            r00              = _mm256_mul_pd(rsq00,rinv00);
            r00              = _mm256_andnot_pd(dummy_mask,r00);

            /* Calculate table index by multiplying r with table scale and truncate to integer */
            rt               = _mm256_mul_pd(r00,vftabscale);
            vfitab           = _mm256_cvttpd_epi32(rt);
            vfeps            = _mm256_sub_pd(rt,_mm256_round_pd(rt, _MM_FROUND_FLOOR));
            vfitab           = _mm_slli_epi32(vfitab,3);

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq00,rinv00);
            felec            = _mm256_mul_pd(velec,rinvsq00);

            /* CUBIC SPLINE TABLE DISPERSION */
            Y                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,0) );
            F                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,1) );
            G                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,2) );
            H                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,3) );
            GMX_MM256_FULLTRANSPOSE4_PD(Y,F,G,H);
            Heps             = _mm256_mul_pd(vfeps,H);
            Fp               = _mm256_add_pd(F,_mm256_mul_pd(vfeps,_mm256_add_pd(G,Heps)));
            FF               = _mm256_add_pd(Fp,_mm256_mul_pd(vfeps,_mm256_add_pd(G,_mm256_add_pd(Heps,Heps))));
            fvdw6            = _mm256_mul_pd(c6_00,FF);

            /* CUBIC SPLINE TABLE REPULSION */
            vfitab           = _mm_add_epi32(vfitab,ifour);
            Y                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,0) );
            F                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,1) );
            G                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,2) );
            H                = _mm256_load_pd( vftab + _mm_extract_epi32(vfitab,3) );
            GMX_MM256_FULLTRANSPOSE4_PD(Y,F,G,H);
            Heps             = _mm256_mul_pd(vfeps,H);
            Fp               = _mm256_add_pd(F,_mm256_mul_pd(vfeps,_mm256_add_pd(G,Heps)));
            FF               = _mm256_add_pd(Fp,_mm256_mul_pd(vfeps,_mm256_add_pd(G,_mm256_add_pd(Heps,Heps))));
            fvdw12           = _mm256_mul_pd(c12_00,FF);
            fvdw             = _mm256_xor_pd(signbit,_mm256_mul_pd(_mm256_add_pd(fvdw6,fvdw12),_mm256_mul_pd(vftabscale,rinv00)));

            fscal            = _mm256_add_pd(felec,fvdw);

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx00);
            ty               = _mm256_mul_pd(fscal,dy00);
            tz               = _mm256_mul_pd(fscal,dz00);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq01,rinv01);
            felec            = _mm256_mul_pd(velec,rinvsq01);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx01);
            ty               = _mm256_mul_pd(fscal,dy01);
            tz               = _mm256_mul_pd(fscal,dz01);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq02,rinv02);
            felec            = _mm256_mul_pd(velec,rinvsq02);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx02);
            ty               = _mm256_mul_pd(fscal,dy02);
            tz               = _mm256_mul_pd(fscal,dz02);

            /* Update vectorial force */
            fix0             = _mm256_add_pd(fix0,tx);
            fiy0             = _mm256_add_pd(fiy0,ty);
            fiz0             = _mm256_add_pd(fiz0,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq10,rinv10);
            felec            = _mm256_mul_pd(velec,rinvsq10);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx10);
            ty               = _mm256_mul_pd(fscal,dy10);
            tz               = _mm256_mul_pd(fscal,dz10);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq11,rinv11);
            felec            = _mm256_mul_pd(velec,rinvsq11);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx11);
            ty               = _mm256_mul_pd(fscal,dy11);
            tz               = _mm256_mul_pd(fscal,dz11);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq12,rinv12);
            felec            = _mm256_mul_pd(velec,rinvsq12);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx12);
            ty               = _mm256_mul_pd(fscal,dy12);
            tz               = _mm256_mul_pd(fscal,dz12);

            /* Update vectorial force */
            fix1             = _mm256_add_pd(fix1,tx);
            fiy1             = _mm256_add_pd(fiy1,ty);
            fiz1             = _mm256_add_pd(fiz1,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq20,rinv20);
            felec            = _mm256_mul_pd(velec,rinvsq20);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx20);
            ty               = _mm256_mul_pd(fscal,dy20);
            tz               = _mm256_mul_pd(fscal,dz20);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx0             = _mm256_add_pd(fjx0,tx);
            fjy0             = _mm256_add_pd(fjy0,ty);
            fjz0             = _mm256_add_pd(fjz0,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq21,rinv21);
            felec            = _mm256_mul_pd(velec,rinvsq21);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx21);
            ty               = _mm256_mul_pd(fscal,dy21);
            tz               = _mm256_mul_pd(fscal,dz21);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx1             = _mm256_add_pd(fjx1,tx);
            fjy1             = _mm256_add_pd(fjy1,ty);
            fjz1             = _mm256_add_pd(fjz1,tz);

            /**************************
             * CALCULATE INTERACTIONS *
             **************************/

            /* COULOMB ELECTROSTATICS */
            velec            = _mm256_mul_pd(qq22,rinv22);
            felec            = _mm256_mul_pd(velec,rinvsq22);

            fscal            = felec;

            fscal            = _mm256_andnot_pd(dummy_mask,fscal);

            /* Calculate temporary vectorial force */
            tx               = _mm256_mul_pd(fscal,dx22);
            ty               = _mm256_mul_pd(fscal,dy22);
            tz               = _mm256_mul_pd(fscal,dz22);

            /* Update vectorial force */
            fix2             = _mm256_add_pd(fix2,tx);
            fiy2             = _mm256_add_pd(fiy2,ty);
            fiz2             = _mm256_add_pd(fiz2,tz);

            fjx2             = _mm256_add_pd(fjx2,tx);
            fjy2             = _mm256_add_pd(fjy2,ty);
            fjz2             = _mm256_add_pd(fjz2,tz);

            fjptrA             = (jnrlistA>=0) ? f+j_coord_offsetA : scratch;
            fjptrB             = (jnrlistB>=0) ? f+j_coord_offsetB : scratch;
            fjptrC             = (jnrlistC>=0) ? f+j_coord_offsetC : scratch;
            fjptrD             = (jnrlistD>=0) ? f+j_coord_offsetD : scratch;

            gmx_mm256_decrement_3rvec_4ptr_swizzle_pd(fjptrA,fjptrB,fjptrC,fjptrD,
                                                      fjx0,fjy0,fjz0,fjx1,fjy1,fjz1,fjx2,fjy2,fjz2);

            /* Inner loop uses 262 flops */
        }

        /* End of innermost loop */

        gmx_mm256_update_iforce_3atom_swizzle_pd(fix0,fiy0,fiz0,fix1,fiy1,fiz1,fix2,fiy2,fiz2,
                                                 f+i_coord_offset,fshift+i_shift_offset);

        /* Increment number of inner iterations */
        inneriter                  += j_index_end - j_index_start;

        /* Outer loop uses 18 flops */
    }

    /* Increment number of outer iterations */
    outeriter        += nri;

    /* Update outer/inner flops */

    inc_nrnb(nrnb,eNR_NBKERNEL_ELEC_VDW_W3W3_F,outeriter*18 + inneriter*262);
}
